#!/usr/bin/env python
"""
  generate jar file from a model file which implements that model
  using a socket interface
"""
# Author: Harco Kuppens


import sys, re, pprint,os  # modules from standard lib (python 2.6 and later)

import tempfile
import shutil
import yaml
import argparse

import util
import uppaal2sutjavafile




def check_dependencies():
    util.check_program('java','.exe')
    util.check_program('javac','.exe')

def args_parsing(args):

    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))

    argparser.add_argument('modelfile', help='input uppaal model file')
    argparser.add_argument('jarfile', help='output jar file implementing model')

    parse_result=argparser.parse_args(args[1:])


    # handle input/output file argument
    modelfile=os.path.abspath(parse_result.modelfile).replace('\\','/')
    jarfile=os.path.abspath(parse_result.jarfile).replace('\\','/')
    # Note: on windows also interprets / as dir sep, and returns path with windows \


    util.check_isreadablefile(modelfile)

    return  [modelfile,jarfile]


def uppaal2jar(modelfile,jarfile):
    """ 
      generate a jar file from a model file which implements that model
      using a socket interface
    """
    sep=os.path.sep

    """
      example of java compilation  :
            mkdir bin
            javac -sourcepath src/:sutsrc/ -cp lib/learnlib-distribution-20110714.jar -d bin  src/sut/Main.java
            jar -cvfm sut.jar manifest.txt -C bin  /

            Where dirs
              - src : source files in sut resource directory
              - sutsrc : tempdir where source of generated sut/Sut.java is located
              - bin : tempdir where binaries are generate
    """

    # 1. generate java source from model
    #------------------------------------

    # src directory to generate sut/Sut.java in
    tmpdir=tempfile.mkdtemp(prefix="sut_uppaal2jar_")
    sutsrc=tmpdir + sep + 'src' + sep
    java_sutfile= sutsrc  + 'sut' + sep + 'Sut.java'
    os.mkdir(sutsrc)
    os.mkdir(sutsrc  + 'sut')

    # create java sut file
    uppaal2sutjavafile.uppaal2sutjavafile(modelfile,java_sutfile)


    # 2. compile all java source
    #----------------------------

    # resource dir is relative from this module :
    resource_dir = os.path.abspath( os.path.dirname(os.path.abspath(__file__)) + '/../resources/' )
    # second src directory which contains also code  which is also used to build jar file
    static_src= resource_dir  + sep +  'sutjar' +  sep + 'src'

    # java sourcepath :
    java_srcpath=static_src + os.path.pathsep + sutsrc

    # java bin directory to put compiled class files in
    java_bindir=tmpdir  + sep + 'bin'  + sep
    os.mkdir(java_bindir)

    # java main file
    java_mainFile = static_src + sep + 'sut' + sep + 'Main.java'
    # manifest file for jar
    jar_manifestfile =  resource_dir + sep +  'sutjar' + sep + 'manifest.txt'


    # use  run_command_with_output and discard the returned output,
    # so any normal output won't be printed to the console
    # note: use cwd='/' to be safe
    util.run_command_with_output([ 'javac', '-sourcepath' , java_srcpath , '-d', java_bindir  , java_mainFile  ],cwd='/')
    util.run_command_with_output([ 'jar', '-cvfm', jarfile , jar_manifestfile , '-C', java_bindir ,  sep ], cwd='/')

    #print "results in directory: " +  tmpdir
    shutil.rmtree(tmpdir)


def main(args):
    """
      generate jar file from a model file which implements that model
      using a socket interface
    """
    check_dependencies()

    (modelfile,jarfile)=args_parsing(args)
    uppaal2jar(modelfile,jarfile)



if __name__ == "__main__":
    print main(sys.argv)

